/**
 * sort entry according to key and order
 *
 * (c) Columbia University, 2001-2004, All Rights Reserved.
 * Author: Weibin Zhao
 */

import java.util.*;

class SortEntry implements Comparable {

    Entry     entry;
    String[]  keys;
    int[]     types;
    int[]     orders;
    Float[]   vals;
    TreeMap   attrtree;

    SortEntry(Entry entry, String[] keys, int[] types, int[] orders, 
              Float[] vals) {
        this.entry    = entry;
        this.keys     = keys;
        this.types    = types;
        this.orders   = orders;
        this.vals     = vals;
        String attr = entry.getAttr(""); // all attributes
        StringTokenizer st = new StringTokenizer(attr, ",");
        int n = st.countTokens();
        attrtree = new TreeMap();
        for (int i=0; i<n; i++) {
            String pair = Util.removeParentheses(st.nextToken());
            StringTokenizer st1 = new StringTokenizer(pair, "=");
            attrtree.put(st1.nextToken(), st1.nextToken());
        }
    }

    public Entry getEntry() {
        return entry;
    }

    public TreeMap getAttrTree() {
        return attrtree;
    }

    public int compareTo(Object o) {
        TreeMap at = ((SortEntry)o).getAttrTree();
        for (int i=0; i<keys.length; i++) {
            String v1   = (String) attrtree.get(keys[i]);
            String v2   = (String) at.get(keys[i]);
            if (v1 == null && v2 == null) {
                continue;
            } else if (v1 == null) {
                if (orders[i] == Const.DecreasingOrder) {
                    return 1;
                } else {
                    return -1;
                }
            } else if (v2 == null) {
                if (orders[i] == Const.DecreasingOrder) {
                    return -1;
                } else {
                    return 1;
                }
            } else {
                if (types[i] == Const.StringSort) {
                    if (v1.compareTo(v2) == 0) continue;
                    if (orders[i] == Const.DecreasingOrder) {
                        return v2.compareTo(v1);        // reverse order
                    } else {
                        return v1.compareTo(v2);
                    }
                } else if (types[i] == Const.NumberSort) {
                    Float v3   = new Float(v1);
                    Float v4   = new Float(v2);
                    if (v3.compareTo(v4) == 0) continue;
                    if (vals[i] != null) { // reference-base
                        float r = vals[i].floatValue();
                        v3 = new Float(Util.abs(v3.floatValue() - r));
                        v4 = new Float(Util.abs(v4.floatValue() - r));
                    }
                    if (orders[i] == Const.DecreasingOrder) {
                        return v4.compareTo(v3);        // reverse order
                    } else {
                        return v3.compareTo(v4);
                    }
                }
            }
        }
        return 0;
    }
}
