/**
 * A generic TCP message handler
 *   (1) send TCP messages
 *   (2) receive TCP message header & body separately
 *   (3) get parameters & close the TCP socket
 *
 * (c) Columbia University, 2001-2004, All Rights Reserved.
 * Author: Weibin Zhao
 */

import java.io.*;
import java.net.*;
import java.util.*;

public class slpTcpHandler {

    da daf;
    Socket socket;
    DataInputStream  in;
    DataOutputStream out;
    byte header[], body[];
    String peer, peer_ip;

    slpTcpHandler(Socket socket, da daf) {
        try {
            this.daf = daf;
            this.socket = socket;
            in  = new DataInputStream(socket.getInputStream());
            out = new DataOutputStream(socket.getOutputStream());
            header = new byte[Const.header_len];
            body = new byte[Const.max_packet];
            peer = socket.getInetAddress().getHostName();
            peer_ip = socket.getInetAddress().getHostAddress();
        } catch (Exception e) {
            System.err.println(e);
        }
    }

    public Socket getSocket() {
        return socket;
    }

    public String getPeerName() {
        return peer;
    }

    public String getPeerIP() {
        return peer_ip;
    }

    public void close() {
        try {
            in.close();
            out.close();
            socket.close();
        } catch (Exception e) {
            System.err.println(e);
        }
    }

    public byte[] getHeader() { // read header
        int len = Const.header_len, start = 0, nread = 0;
        try {
            while (len > 0) {
                nread = in.read(header, start, len);
                if (nread <= 0) { // EOF means the connection has been closed
                    if (daf != null) daf.checkPeerConn(peer, socket);
                    return null;
                }
                start += nread;
                len   -= nread;
            }
        } catch (InterruptedIOException e) { // TimeOut
            System.err.println(e.getMessage());
            return null;
        } catch (SocketException e) { // keep silent
        } catch (Exception e) {
            System.err.println(e);
        }
        return header;
    }

    public byte[] getBody(int packet_len) {
        int len = packet_len - Const.header_len, start = 0, nread = 0;
        if (len > body.length) {
            body = new byte[len];       // in cast we received a big packet
        }
        try {
            while (len > 0) {
                nread = in.read(body, start, len);
                start += nread;
                len   -= nread;
            }
        } catch (Exception e) {
            System.err.println(e);
            return null;
        }
        return body;
    }

    public void send(byte[] buf, int len) {
        try {
            out.write(buf, 0, len);
            out.flush();
        } catch (IOException e) {
            System.err.println(e.getMessage());
        } catch (Exception e) {
            System.err.println(e);
        }
    }
}
